@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2009 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2009-2010 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -105,6 +105,13 @@ typedef enum {
 	GCM_CALIBRATE_PRINT_KIND_UNKNOWN
 } GcmCalibratePrintKind;
 
+typedef enum {
+	GCM_CALIBRATE_PRECISION_SHORT,
+	GCM_CALIBRATE_PRECISION_NORMAL,
+	GCM_CALIBRATE_PRECISION_LONG,
+	GCM_CALIBRATE_PRECISION_UNKNOWN
+} GcmCalibratePrecision;
+
 GType		 gcm_calibrate_get_type			(void);
 GcmCalibrate	*gcm_calibrate_new			(void);
 gboolean	 gcm_calibrate_display			(GcmCalibrate	*calibrate,
@@ -126,6 +133,7 @@ const gchar	*gcm_calibrate_get_model_fallback	(GcmCalibrate	*calibrate);
 const gchar	*gcm_calibrate_get_description_fallback	(GcmCalibrate	*calibrate);
 const gchar	*gcm_calibrate_get_manufacturer_fallback (GcmCalibrate	*calibrate);
 const gchar	*gcm_calibrate_get_device_fallback	(GcmCalibrate	*calibrate);
+const gchar	*gcm_calibrate_get_filename_result	(GcmCalibrate	*calibrate);
 
 G_END_DECLS
 
