@@ -70,6 +70,8 @@ struct _GcmClientPrivate
 	gboolean			 loading;
 	guint				 loading_refcount;
 	gboolean			 use_threads;
+	gboolean			 init_cups;
+	gboolean			 init_sane;
 };
 
 enum {
@@ -131,6 +133,22 @@ gcm_client_done_loading (GcmClient *client)
 	g_static_mutex_unlock (&mutex);
 }
 
+/**
+ * gcm_client_add_loading:
+ **/
+static void
+gcm_client_add_loading (GcmClient *client)
+{
+	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
+
+	/* decrement refcount, with a lock */
+	g_static_mutex_lock (&mutex);
+	client->priv->loading_refcount++;
+	if (client->priv->loading_refcount > 0)
+		gcm_client_set_loading (client, TRUE);
+	g_static_mutex_unlock (&mutex);
+}
+
 /**
  * gcm_client_get_devices:
  *
@@ -646,6 +664,10 @@ gcm_client_add_connected_devices_xrandr (GcmClient *client, GError **error)
 		return FALSE;
 	for (i=0; outputs[i] != NULL; i++)
 		gcm_client_xrandr_add (client, outputs[i]);
+
+	/* inform the UI */
+	gcm_client_done_loading (client);
+
 	return TRUE;
 }
 
@@ -702,6 +724,14 @@ gcm_client_add_connected_devices_cups (GcmClient *client, GError **error)
 	gint i;
 	GcmClientPrivate *priv = client->priv;
 
+	/* initialize */
+	if (!client->priv->init_cups) {
+		httpInitialize();
+		/* should be okay for localhost */
+		client->priv->http = httpConnectEncrypt (cupsServer (), ippPort (), cupsEncryption ());
+		client->priv->init_cups = TRUE;
+	}
+
 	num_dests = cupsGetDests2 (priv->http, &dests);
 	egg_debug ("got %i printers", num_dests);
 
@@ -785,17 +815,28 @@ static gboolean
 gcm_client_add_connected_devices_sane (GcmClient *client, GError **error)
 {
 	gint i;
+	gboolean ret = TRUE;
 	SANE_Status status;
 	const SANE_Device **device_list;
 
 	/* force sane to drop it's cache of devices -- yes, it is that crap */
-	sane_exit ();
-	sane_init (NULL, NULL);
+	if (client->priv->init_sane) {
+		sane_exit ();
+		client->priv->init_sane = FALSE;
+	}
+	status = sane_init (NULL, NULL);
+	if (status != SANE_STATUS_GOOD) {
+		ret = FALSE;
+		g_set_error (error, 1, 0, "failed to init SANE: %s", sane_strstatus (status));
+		goto out;
+	}
+	client->priv->init_sane = TRUE;
 
 	/* get scanners on the local server */
 	status = sane_get_devices (&device_list, FALSE);
 	if (status != SANE_STATUS_GOOD) {
-		egg_warning ("failed to get devices from SANE: %s", sane_strstatus (status));
+		ret = FALSE;
+		g_set_error (error, 1, 0, "failed to get devices from SANE: %s", sane_strstatus (status));
 		goto out;
 	}
 
@@ -811,7 +852,7 @@ gcm_client_add_connected_devices_sane (GcmClient *client, GError **error)
 out:
 	/* inform the UI */
 	gcm_client_done_loading (client);
-	return TRUE;
+	return ret;
 }
 
 /**
@@ -966,53 +1007,68 @@ out:
  * gcm_client_add_connected:
  **/
 gboolean
-gcm_client_add_connected (GcmClient *client, GError **error)
+gcm_client_add_connected (GcmClient *client, GcmClientColdplug coldplug, GError **error)
 {
 	gboolean ret;
 	GThread *thread;
 
 	g_return_val_if_fail (GCM_IS_CLIENT (client), FALSE);
 
+	/* reset */
+	client->priv->loading_refcount = 0;
+
 	/* XRandR */
-	ret = gcm_client_add_connected_devices_xrandr (client, error);
-	if (!ret)
-		goto out;
-
-	/* inform UI if we are loading devices still */
-	client->priv->loading_refcount = 3;
-	gcm_client_set_loading (client, TRUE);
-
-	/* UDEV */
-	if (client->priv->use_threads) {
-		thread = g_thread_create ((GThreadFunc) gcm_client_add_connected_devices_udev_thrd, client, FALSE, error);
-		if (thread == NULL)
-			goto out;
-	} else {
-		ret = gcm_client_add_connected_devices_udev (client, error);
+	if (!coldplug || coldplug & GCM_CLIENT_COLDPLUG_XRANDR) {
+		gcm_client_add_loading (client);
+		egg_debug ("adding devices of type XRandR");
+		ret = gcm_client_add_connected_devices_xrandr (client, error);
 		if (!ret)
 			goto out;
 	}
 
+	/* UDEV */
+	if (!coldplug || coldplug & GCM_CLIENT_COLDPLUG_UDEV) {
+		gcm_client_add_loading (client);
+		egg_debug ("adding devices of type UDEV");
+		if (client->priv->use_threads) {
+			thread = g_thread_create ((GThreadFunc) gcm_client_add_connected_devices_udev_thrd, client, FALSE, error);
+			if (thread == NULL)
+				goto out;
+		} else {
+			ret = gcm_client_add_connected_devices_udev (client, error);
+			if (!ret)
+				goto out;
+		}
+	}
+
 	/* CUPS */
-	if (client->priv->use_threads) {
-		thread = g_thread_create ((GThreadFunc) gcm_client_add_connected_devices_cups_thrd, client, FALSE, error);
-		if (thread == NULL)
-			goto out;
-	} else {
-		ret = gcm_client_add_connected_devices_cups (client, error);
-		if (!ret)
-			goto out;
+	if (!coldplug || coldplug & GCM_CLIENT_COLDPLUG_CUPS) {
+		gcm_client_add_loading (client);
+		egg_debug ("adding devices of type CUPS");
+		if (client->priv->use_threads) {
+			thread = g_thread_create ((GThreadFunc) gcm_client_add_connected_devices_cups_thrd, client, FALSE, error);
+			if (thread == NULL)
+				goto out;
+		} else {
+			ret = gcm_client_add_connected_devices_cups (client, error);
+			if (!ret)
+				goto out;
+		}
 	}
 
 	/* SANE */
-	if (client->priv->use_threads) {
-		thread = g_thread_create ((GThreadFunc) gcm_client_add_connected_devices_sane_thrd, client, FALSE, error);
-		if (thread == NULL)
-			goto out;
-	} else {
-		ret = gcm_client_add_connected_devices_sane (client, error);
-		if (!ret)
-			goto out;
+	if (!coldplug || coldplug & GCM_CLIENT_COLDPLUG_SANE) {
+		gcm_client_add_loading (client);
+		egg_debug ("adding devices of type SANE");
+		if (client->priv->use_threads) {
+			thread = g_thread_create ((GThreadFunc) gcm_client_add_connected_devices_sane_thrd, client, FALSE, error);
+			if (thread == NULL)
+				goto out;
+		} else {
+			ret = gcm_client_add_connected_devices_sane (client, error);
+			if (!ret)
+				goto out;
+		}
 	}
 out:
 	return ret;
@@ -1289,12 +1345,13 @@ static void
 gcm_client_init (GcmClient *client)
 {
 	const gchar *subsystems[] = {"usb", "video4linux", NULL};
-	SANE_Status status;
 
 	client->priv = GCM_CLIENT_GET_PRIVATE (client);
 	client->priv->display_name = NULL;
 	client->priv->loading_refcount = 0;
 	client->priv->use_threads = FALSE;
+	client->priv->init_cups = FALSE;
+	client->priv->init_sane = FALSE;
 	client->priv->array = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	client->priv->screen = gcm_screen_new ();
 	g_signal_connect (client->priv->screen, "outputs-changed",
@@ -1304,17 +1361,6 @@ gcm_client_init (GcmClient *client)
 	client->priv->gudev_client = g_udev_client_new (subsystems);
 	g_signal_connect (client->priv->gudev_client, "uevent",
 			  G_CALLBACK (gcm_client_uevent_cb), client);
-
-	/* for CUPS */
-	httpInitialize();
-
-	/* for SANE */
-	status = sane_init (NULL, NULL);
-	if (status != SANE_STATUS_GOOD)
-		egg_warning ("failed to init SANE: %s", sane_strstatus (status));
-
-	/* should be okay for localhost */
-	client->priv->http = httpConnectEncrypt (cupsServer (), ippPort (), cupsEncryption ());
 }
 
 /**
@@ -1338,8 +1384,10 @@ gcm_client_finalize (GObject *object)
 	g_ptr_array_unref (priv->array);
 	g_object_unref (priv->gudev_client);
 	g_object_unref (priv->screen);
-	httpClose (priv->http);
-	sane_exit ();
+	if (client->priv->init_cups)
+		httpClose (priv->http);
+	if (client->priv->init_sane)
+		sane_exit ();
 
 	G_OBJECT_CLASS (gcm_client_parent_class)->finalize (object);
 }
