@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2009 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2009-2010 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -314,7 +314,7 @@ gcm_parser_load_icc_vcgt (GcmProfileLcms1 *profile_lcms1, const guint8 *data, gu
 	}
 
 	/* we didn't understand the encoding */
-	egg_warning ("gamma type encoding not recognised");
+	egg_warning ("gamma type encoding not recognized");
 out:
 	return ret;
 }
@@ -396,9 +396,9 @@ gcm_profile_lcms1_parse_multi_localized_unicode (GcmProfileLcms1 *profile_lcms1,
 		goto out;
 	}
 
-	/* an unrecognised tag */
+	/* an unrecognized tag */
 	for (i=0x0; i<0x1c; i++) {
-		egg_warning ("unrecognised text tag");
+		egg_warning ("unrecognized text tag");
 		if (data[i] >= 'A' && data[i] <= 'z')
 			egg_debug ("%i\t%c (%i)", i, data[i], data[i]);
 		else
@@ -435,8 +435,8 @@ gcm_profile_lcms1_parse_data (GcmProfile *profile, const guint8 *data, gsize len
 	guint tag_offset;
 	icProfileClassSignature profile_class;
 	icColorSpaceSignature color_space;
-	GcmColorspaceEnum colorspace;
-	GcmProfileTypeEnum profile_type;
+	GcmColorspace colorspace;
+	GcmProfileKind profile_kind;
 	cmsCIEXYZ cie_xyz;
 	cmsCIEXYZTRIPLE cie_illum;
 	struct tm created;
@@ -470,7 +470,7 @@ gcm_profile_lcms1_parse_data (GcmProfile *profile, const guint8 *data, gsize len
 			      "cie-z", cie_xyz.Z,
 			      NULL);
 		g_object_set (profile,
-			      "white-point", xyz,
+			      "white", xyz,
 			      NULL);
 		g_object_unref (xyz);
 	} else {
@@ -487,83 +487,79 @@ gcm_profile_lcms1_parse_data (GcmProfile *profile, const guint8 *data, gsize len
 			      "cie-z", cie_xyz.Z,
 			      NULL);
 		g_object_set (profile,
-			      "black-point", xyz,
+			      "black", xyz,
 			      NULL);
 		g_object_unref (xyz);
 	} else {
 		egg_warning ("failed to get black point");
 	}
 
-	/* get the profile_lcms1 type */
+	/* get the profile kind */
 	profile_class = cmsGetDeviceClass (priv->lcms_profile);
 	switch (profile_class) {
 	case icSigInputClass:
-		profile_type = GCM_PROFILE_TYPE_ENUM_INPUT_DEVICE;
+		profile_kind = GCM_PROFILE_KIND_INPUT_DEVICE;
 		break;
 	case icSigDisplayClass:
-		profile_type = GCM_PROFILE_TYPE_ENUM_DISPLAY_DEVICE;
+		profile_kind = GCM_PROFILE_KIND_DISPLAY_DEVICE;
 		break;
 	case icSigOutputClass:
-		profile_type = GCM_PROFILE_TYPE_ENUM_OUTPUT_DEVICE;
+		profile_kind = GCM_PROFILE_KIND_OUTPUT_DEVICE;
 		break;
 	case icSigLinkClass:
-		profile_type = GCM_PROFILE_TYPE_ENUM_DEVICELINK;
+		profile_kind = GCM_PROFILE_KIND_DEVICELINK;
 		break;
 	case icSigColorSpaceClass:
-		profile_type = GCM_PROFILE_TYPE_ENUM_COLORSPACE_CONVERSION;
+		profile_kind = GCM_PROFILE_KIND_COLORSPACE_CONVERSION;
 		break;
 	case icSigAbstractClass:
-		profile_type = GCM_PROFILE_TYPE_ENUM_ABSTRACT;
+		profile_kind = GCM_PROFILE_KIND_ABSTRACT;
 		break;
 	case icSigNamedColorClass:
-		profile_type = GCM_PROFILE_TYPE_ENUM_NAMED_COLOR;
+		profile_kind = GCM_PROFILE_KIND_NAMED_COLOR;
 		break;
 	default:
-		profile_type = GCM_PROFILE_TYPE_ENUM_UNKNOWN;
+		profile_kind = GCM_PROFILE_KIND_UNKNOWN;
 	}
-	g_object_set (profile,
-		      "type", profile_type,
-		      NULL);
+	gcm_profile_set_kind (profile, profile_kind);
 
 	/* get colorspace */
 	color_space = cmsGetColorSpace (priv->lcms_profile);
 	switch (color_space) {
 	case icSigXYZData:
-		colorspace = GCM_COLORSPACE_ENUM_XYZ;
+		colorspace = GCM_COLORSPACE_XYZ;
 		break;
 	case icSigLabData:
-		colorspace = GCM_COLORSPACE_ENUM_LAB;
+		colorspace = GCM_COLORSPACE_LAB;
 		break;
 	case icSigLuvData:
-		colorspace = GCM_COLORSPACE_ENUM_LUV;
+		colorspace = GCM_COLORSPACE_LUV;
 		break;
 	case icSigYCbCrData:
-		colorspace = GCM_COLORSPACE_ENUM_YCBCR;
+		colorspace = GCM_COLORSPACE_YCBCR;
 		break;
 	case icSigYxyData:
-		colorspace = GCM_COLORSPACE_ENUM_YXY;
+		colorspace = GCM_COLORSPACE_YXY;
 		break;
 	case icSigRgbData:
-		colorspace = GCM_COLORSPACE_ENUM_RGB;
+		colorspace = GCM_COLORSPACE_RGB;
 		break;
 	case icSigGrayData:
-		colorspace = GCM_COLORSPACE_ENUM_GRAY;
+		colorspace = GCM_COLORSPACE_GRAY;
 		break;
 	case icSigHsvData:
-		colorspace = GCM_COLORSPACE_ENUM_HSV;
+		colorspace = GCM_COLORSPACE_HSV;
 		break;
 	case icSigCmykData:
-		colorspace = GCM_COLORSPACE_ENUM_CMYK;
+		colorspace = GCM_COLORSPACE_CMYK;
 		break;
 	case icSigCmyData:
-		colorspace = GCM_COLORSPACE_ENUM_CMY;
+		colorspace = GCM_COLORSPACE_CMY;
 		break;
 	default:
-		colorspace = GCM_COLORSPACE_ENUM_UNKNOWN;
+		colorspace = GCM_COLORSPACE_UNKNOWN;
 	}
-	g_object_set (profile,
-		      "colorspace", colorspace,
-		      NULL);
+	gcm_profile_set_colorspace (profile, colorspace);
 
 	/* get primary illuminants */
 	ret = cmsTakeColorants (&cie_illum, priv->lcms_profile);
@@ -614,7 +610,7 @@ gcm_profile_lcms1_parse_data (GcmProfile *profile, const guint8 *data, gsize len
 			      "cie-z", cie_illum.Red.Z,
 			      NULL);
 		g_object_set (profile,
-			      "luminance_red", xyz,
+			      "red", xyz,
 			      NULL);
 		g_object_unref (xyz);
 
@@ -626,7 +622,7 @@ gcm_profile_lcms1_parse_data (GcmProfile *profile, const guint8 *data, gsize len
 			      "cie-z", cie_illum.Green.Z,
 			      NULL);
 		g_object_set (profile,
-			      "luminance-green", xyz,
+			      "green", xyz,
 			      NULL);
 		g_object_unref (xyz);
 
@@ -638,7 +634,7 @@ gcm_profile_lcms1_parse_data (GcmProfile *profile, const guint8 *data, gsize len
 			      "cie-z", cie_illum.Blue.Z,
 			      NULL);
 		g_object_set (profile,
-			      "luminance-blue", xyz,
+			      "blue", xyz,
 			      NULL);
 		g_object_unref (xyz);
 	} else {
@@ -649,9 +645,7 @@ gcm_profile_lcms1_parse_data (GcmProfile *profile, const guint8 *data, gsize len
 	ret = cmsTakeCreationDateTime (&created, priv->lcms_profile);
 	if (ret) {
 		text = gcm_utils_format_date_time (&created);
-		g_object_set (profile,
-			      "datetime", text,
-			      NULL);
+		gcm_profile_set_datetime (profile, text);
 		g_free (text);
 	}
 
@@ -670,22 +664,22 @@ gcm_profile_lcms1_parse_data (GcmProfile *profile, const guint8 *data, gsize len
 
 		if (tag_id == icSigProfileDescriptionTag) {
 			text = gcm_profile_lcms1_parse_multi_localized_unicode (profile_lcms1, data + tag_offset, tag_size);
-			g_object_set (profile, "description", text, NULL);
+			gcm_profile_set_description (profile, text);
 			g_free (text);
 		}
 		if (tag_id == icSigCopyrightTag) {
 			text = gcm_profile_lcms1_parse_multi_localized_unicode (profile_lcms1, data + tag_offset, tag_size);
-			g_object_set (profile, "copyright", text, NULL);
+			gcm_profile_set_copyright (profile, text);
 			g_free (text);
 		}
 		if (tag_id == icSigDeviceMfgDescTag) {
 			text = gcm_profile_lcms1_parse_multi_localized_unicode (profile_lcms1, data + tag_offset, tag_size);
-			g_object_set (profile, "manufacturer", text, NULL);
+			gcm_profile_set_manufacturer (profile, text);
 			g_free (text);
 		}
 		if (tag_id == icSigDeviceModelDescTag) {
 			text = gcm_profile_lcms1_parse_multi_localized_unicode (profile_lcms1, data + tag_offset, tag_size);
-			g_object_set (profile, "model", text, NULL);
+			gcm_profile_set_model (profile, text);
 			g_free (text);
 		}
 		if (tag_id == icSigMachineLookUpTableTag) {
@@ -712,9 +706,7 @@ gcm_profile_lcms1_parse_data (GcmProfile *profile, const guint8 *data, gsize len
 	ret = TRUE;
 
 	/* set properties */
-	g_object_set (profile,
-		      "has-vcgt", priv->has_vcgt_formula || priv->has_vcgt_table,
-		      NULL);
+	gcm_profile_set_has_vcgt (profile, priv->has_vcgt_formula || priv->has_vcgt_table);
 
 	egg_debug ("Has MLUT:         %s", priv->has_mlut ? "YES" : "NO");
 	egg_debug ("Has VCGT formula: %s", priv->has_vcgt_formula ? "YES" : "NO");
@@ -870,17 +862,13 @@ gcm_profile_lcms1_generate_curve (GcmProfile *profile, guint size)
 	cmsHPROFILE srgb_profile_lcms1 = NULL;
 	cmsHTRANSFORM transform = NULL;
 	guint type;
-	GcmColorspaceEnum colorspace;
+	GcmColorspace colorspace;
 	GcmProfileLcms1 *profile_lcms1 = GCM_PROFILE_LCMS1 (profile);
 	GcmProfileLcms1Private *priv = profile_lcms1->priv;
 
-	/* get data */
-	g_object_get (profile,
-		      "colorspace", &colorspace,
-		      NULL);
-
-	/* run through the profile_lcms1 */
-	if (colorspace == GCM_COLORSPACE_ENUM_RGB) {
+	/* run through the profile */
+	colorspace = gcm_profile_get_colorspace (profile);
+	if (colorspace == GCM_COLORSPACE_RGB) {
 
 		/* RGB */
 		component_width = 3;
